using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Mvc.Ajax;
using EPiServer.Shell.Gadgets;
using EPiServer.Shell.Web;
using EPiServer.Core;
using EPiServer.XForms;
using EPiServer.Models;
using EPiServer.Data.Dynamic;
using System.Collections.Specialized;
using System.Text;
using EPiServer.XForms.WebControls;
using EPiServer.Shell.Gadgets.Contracts;
using EPiServer.Models.Poll;
using EPiServerConfiguration = EPiServer.Configuration;
using EPiServer.Shell.Resources;
using EPiServer.Shell.UI;
using EPiServer.XForms.Util;
using System.Xml.Linq;
using System.IO;


namespace EPiServer.Controllers
{
    [Gadget]    
    public class PollController : Controller
    {
        //Poll gadget
        //Author: Johan "Swippen" Sellberg - johan@sockerbit.net

        private const string googleChartUrl = "http://chart.apis.google.com/chart?";

        private DynamicDataStore<Settings> _gadgetSettingsStore;

        /// <summary>
        /// The default Index Action that loads model data for the Index view
        /// </summary>
        /// <returns>The Index ActionResult</returns>
        public ActionResult Index(Guid gadgetId)
        {
            PollModel model = GetModel(gadgetId);
            return View(model);
        }

        private PollModel GetModel(Guid gadgetId)
        {
            Settings settings = LoadSettings(gadgetId);
            
            return GetModel(gadgetId,settings);
        }

        /// <summary>
        /// Gets the model by loading the XForm and extracting data.
        /// </summary>
        /// <param name="gadgetId">The gadget id.</param>
        /// <param name="settings">The settings.</param>
        /// <returns></returns>
        private PollModel GetModel(Guid gadgetId, Settings settings)
        {
            
            StringBuilder link = null;
            XForm form = null;
            PollModel model = new PollModel();
            try
            {
                PageData data = EPiServer.DataFactory.Instance.GetPage(new PageReference(settings.PageId));
                PropertyData propData = data.Property[settings.XFormPropertyName];
                form = XForm.CreateInstance(new Guid((String)propData.Value));
                SerializableXmlDocument doc = form.Document;
                XDocument xmlDoc = XDocument.Parse(doc.InnerXml);
                string heading = String.Empty;
                string question = String.Empty;
                //Try get the heading. If no heading are found we return a default heading with a message whats wrong.
                try
                {
                    var headingQuery = from q in xmlDoc.Descendants("span")
                            where q.Attribute("class").Value.Contains("heading")
                            select new { heading = q.Value };
                    heading = (string)(headingQuery.ElementAt(0).heading);
                }
                catch (Exception)
                {
                    heading = "No heading with cssclass heading found.";
                }                

                //Try to get the question. If no question are found we return a space character only. (We dont display any question)
                try
                {
                    var questionQuery = from q in xmlDoc.Descendants()
                                        where (q.Attribute("ref") != null) && q.Attribute("ref").Value.Equals(settings.FieldNameInXForm)
                                        select new { question = q.Attribute("title").Value, reff = q.Attribute("ref").Value };

                    question = (string)(questionQuery.ElementAt(0).question);
                }
                catch (Exception)
                {
                    question = "&nbsp;";
                }
                model.PollHeading = heading;
                model.PollQuestion = question;
                link = GetChartLink(settings, form);                
            }
            //If we have no posts in the XForm we want to display it in a nice way.
            catch (NoXFormDataException e)
            {
                link = new StringBuilder();
                link.Append(googleChartUrl + "chs=520x120&cht=p3&chd=t:100&chl=");
                link.Append(e.Message);
                if (String.IsNullOrEmpty(model.PollHeading))
                {
                    model.PollHeading = "No Data";
                }
                
            }
            //This is so that if we have bad configuration (cant find page o form or so) We dont want to crash the entire dashboad.
            catch (Exception)
            {
                link = new StringBuilder();
                link.Append(googleChartUrl + "chs=520x120&cht=p3&chd=t:100&chl=");
                if (String.IsNullOrEmpty(model.PollHeading))
                {
                    model.PollHeading = "Exception in Chart Gadget";
                }
            }

            model.GadgetId = gadgetId;
            model.Link = link.ToString();
            model.LiveUpdate = settings.LiveUpdate;
            return model;
        }

        /// <summary>
        /// Gets the chart link by traversing the XForm data and returning a link with the result
        /// </summary>
        /// <param name="settings">The settings.</param>
        /// <param name="form">The form.</param>
        /// <returns></returns>
        private static StringBuilder GetChartLink(Settings settings, XForm form)
        {
            StringBuilder link = new StringBuilder();
            link.Append(googleChartUrl + "chs=520x120&amp;cht=");
            link.Append(settings.ChartType);
            StringBuilder value = new StringBuilder();
            StringBuilder titles = new StringBuilder();
            Dictionary<string, int> results = new Dictionary<string, int>();
            int totalCount = 0;


            foreach (XFormData formData in form.GetPostedData())
            {
                NameValueCollection values = formData.GetValues();
                string key = values.Get(settings.FieldNameInXForm);
                if (results.Keys.Contains(key))
                {
                    results[key]++;
                }
                else
                {
                    results.Add(values.Get(settings.FieldNameInXForm), 1);
                }
                totalCount++;
            }

            if (totalCount == 0)
            {
                throw new NoXFormDataException();
            }

            int index = 0;
            if (results.Count > 0)
            {
                foreach (KeyValuePair<String, int> pair in results)
                {
                    if (index > 0)
                    {
                        value.Append(",");
                        titles.Append("|");
                    }
                    int percent = Convert.ToInt32((float)((float)pair.Value / (float)totalCount) * 100);
                    value.Append(percent.ToString());
                    titles.Append(String.Format("{0} ({1})", pair.Key, pair.Value));
                    index++;
                }
            }
            link.Append(String.Format("&amp;chd=t:{0}", value.ToString()));
            link.Append(String.Format("&amp;chl={0}", titles.ToString()));
            return link;
        }

        /// <summary>
        /// The Configure action that is used to set up the Configure view.
        /// </summary>
        /// <param name="gadgetId">The gadget id.</param>
        /// <returns>The Configure ActionResult</returns>
        [GadgetAction(ActionType.Menu, TextResourceKey = "Edit", ResourceType = typeof(EPiServer.Shell.UI.Views.Shared.SharedResources))]
        public ActionResult Configure(Guid gadgetId)
        {            
            Settings settings = LoadSettings(gadgetId);

            PollSettingsData data = new PollSettingsData();
            data.Settings = settings;
            data.GadgetId = gadgetId;

            return View("Configure", data);
        }

        /// <summary>
        /// Action that generates a link from the data stored for the gadget.
        /// </summary>
        /// <param name="gadgetId">The gadget id.</param>
        /// <returns>A json object with a link</returns>
        [AcceptVerbs(HttpVerbs.Post)]
        public JsonResult GetLink(Guid gadgetId)
        {
            Settings settings = LoadSettings(gadgetId);
            StringBuilder link = null;                     
            try
            {
                PageData data = EPiServer.DataFactory.Instance.GetPage(new PageReference(settings.PageId));
                PropertyData propData = data.Property[settings.XFormPropertyName];
                XForm form = XForm.CreateInstance(new Guid((String)propData.Value));

                link = GetChartLink(settings, form);
            }
            //If we have no posts in the XForm we want to display it in a nice way.
            catch (NoXFormDataException e)
            {
                link = new StringBuilder();
                link.Append(googleChartUrl + "chs=520x120&cht=p3&chd=t:100&chl=");
                link.Append(e.Message);
                
            }
            //This is so that if we have bad configuration (cant find page o form or so) We dont want to crash the entire dashboad.
            catch (Exception)
            {
                link = new StringBuilder();
                link.Append(googleChartUrl + "chs=520x120&cht=p3&chd=t:100&chl=");                
            }

            return Json(new {link = link.ToString()});
        }

        /// <summary>
        /// The action used when saving settings for the gadget
        /// </summary>
        /// <param name="settings">The settings.</param>
        /// <param name="gadgetId">The gadget id.</param>
        /// <returns>The Index ActionResult.</returns>
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult SaveSettings(Settings settings, Guid gadgetId)
        {
            SaveSettings(gadgetId, settings);

            PollModel model = GetModel(gadgetId, settings);
            return View("Index",model);
        }

        /// <summary>
        /// Utilitymethod to save the settings.
        /// </summary>
        /// <param name="gadgetId">The gadget id.</param>
        /// <param name="settings">The settings.</param>
        private void SaveSettings(Guid gadgetId, Settings settings)
        {
            Settings savedSettings = LoadSettings(gadgetId);
            settings.GadgetId = gadgetId;
            settings.Id = savedSettings.Id;
            GadgetSettingsStore.Save(settings);
        }


        /// <summary>
        /// Utilitymethod to load the settings.
        /// </summary>
        /// <param name="gadgetId">The gadget id.</param>
        /// <returns>The Settings object</returns>
        private Settings LoadSettings(Guid gadgetId)
        {
            Settings settings = GadgetSettingsStore.Find("GadgetId", gadgetId).FirstOrDefault();
            if (settings == null)
            {
                settings = new Settings();
            }

            return settings;
        }        

        /// <summary>
        /// A store to handle the settings for the Poll gadgets.
        /// </summary>
        public DynamicDataStore<Settings> GadgetSettingsStore
        {
            get
            {
                if (_gadgetSettingsStore == null)
                {
                    _gadgetSettingsStore = DynamicDataStore<Settings>.CreateStore("PollStore", false);
                }
                return _gadgetSettingsStore;
            }
        }
    }
    public class NoXFormDataException : System.Exception
    {
        // Custom exception that is thrown when no ata is found for the gadget.

        public NoXFormDataException(): base("No poll data found for this XForm.")
        {
        }

        public NoXFormDataException(string message)
            : base(message)
        {
        }
    }
}
